@@ -7,6 +7,11 @@
 ;;   You must not remove this notice, or any other, from this software.
 
 (ns clojure.core.async
+  "Facilities for async programming and communication.
+
+go blocks are dispatched over an internal thread pool, which
+defaults to 8 threads. The size of this pool can be modified using
+the Java system property `clojure.core.async.pool-size`."
   (:refer-clojure :exclude [reduce into merge map take partition
                             partition-by] :as core)
   (:require [clojure.core.async.impl.protocols :as impl]
@@ -25,19 +30,22 @@
 
 (alias 'core 'clojure.core)
 
-(set! *warn-on-reflection* true)
+(set! *warn-on-reflection* false)
 
 (defn fn-handler
-  [f]
-  (reify
-   Lock
-   (lock [_])
-   (unlock [_])
+  ([f]
+   (fn-handler f true))
+  ([f blockable]
+   (reify
+     Lock
+     (lock [_])
+     (unlock [_])
 
-   impl/Handler
-   (active? [_] true)
-   (lock-id [_] 0)
-   (commit [_] f)))
+     impl/Handler
+     (active? [_] true)
+     (blockable? [_] blockable)
+     (lock-id [_] 0)
+     (commit [_] f))))
 
 (defn buffer
   "Returns a fixed buffer of size n. When full, puts will block/park."
@@ -79,6 +87,17 @@
      (when xform (assert buf-or-n "buffer must be supplied when transducer is"))
      (channels/chan (if (number? buf-or-n) (buffer buf-or-n) buf-or-n) xform ex-handler)))
 
+(defn promise-chan
+  "Creates a promise channel with an optional transducer, and an optional
+  exception-handler. A promise channel can take exactly one value that consumers
+  will receive. Once full, puts complete but val is dropped (no transfer).
+  Consumers will block until either a value is placed in the channel or the
+  channel is closed. See chan for the semantics of xform and ex-handler."
+  ([] (promise-chan nil))
+  ([xform] (promise-chan xform nil))
+  ([xform ex-handler]
+     (chan (buffers/promise-buffer) xform ex-handler)))
+
 (defn timeout
   "Returns a channel that will close after msecs"
   [^long msecs]
@@ -137,7 +156,7 @@
 
 (defn put!
   "Asynchronously puts a val into port, calling fn1 (if supplied) when
-   complete, passing true iff port is already closed. nil values are
+   complete, passing false iff port is already closed. nil values are
    not allowed. If on-caller? (default true) is true, and the put is
    immediately accepted, will call fn1 on calling thread.  Returns
    true unless port is already closed."
@@ -194,6 +213,7 @@
 
      impl/Handler
      (active? [_] @flag)
+     (blockable? [_] true)
      (lock-id [_] id)
      (commit [_]
              (reset! flag nil)
@@ -207,6 +227,7 @@
 
      impl/Handler
      (active? [_] (impl/active? flag))
+     (blockable? [_] true)
      (lock-id [_] (impl/lock-id flag))
      (commit [_]
              (impl/commit flag)
@@ -365,6 +386,19 @@
     (ioc/aset-all! state ioc/VALUE-IDX @cb)
     :recur))
 
+(defn offer!
+  "Puts a val into port if it's possible to do so immediately.
+   nil values are not allowed. Never blocks. Returns true if offer succeeds."
+  [port val]
+  (let [ret (impl/put! port val (fn-handler nop false))]
+    (when ret @ret)))
+
+(defn poll!
+  "Takes a val from port if it's possible to do so immediately.
+   Never blocks. Returns value if successful, nil otherwise."
+  [port]
+  (let [ret (impl/take! port (fn-handler nop false))]
+    (when ret @ret)))
 
 (defmacro go
   "Asynchronously executes the body, returning immediately to the
@@ -568,7 +602,10 @@
     (let [v (<! ch)]
       (if (nil? v)
         ret
-        (recur (f ret v))))))
+        (let [ret' (f ret v)]
+          (if (reduced? ret')
+            @ret'
+            (recur ret')))))))
 
 (defn- bounded-count
   "Returns the smaller of n or the count of coll, without examining
@@ -943,7 +980,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; deprecated - do not use ;;;;;;;;;;;;;;;;;;;;;;;;;
 (defn map<
-  "Deprecated - this function will be removed. Use transformer instead"
+  "Deprecated - this function will be removed. Use transducer instead"
   [f ch]
   (reify
    impl/Channel
@@ -961,6 +998,7 @@
 
           impl/Handler
           (active? [_] (impl/active? fn1))
+          (blockable? [_] true)
           (lock-id [_] (impl/lock-id fn1))
           (commit [_]
            (let [f1 (impl/commit fn1)]
@@ -973,7 +1011,7 @@
    (put! [_ val fn1] (impl/put! ch val fn1))))
 
 (defn map>
-  "Deprecated - this function will be removed. Use transformer instead"
+  "Deprecated - this function will be removed. Use transducer instead"
   [f ch]
   (reify
    impl/Channel
@@ -988,7 +1026,7 @@
     (impl/put! ch (f val) fn1))))
 
 (defn filter>
-  "Deprecated - this function will be removed. Use transformer instead"
+  "Deprecated - this function will be removed. Use transducer instead"
   [p ch]
   (reify
    impl/Channel
@@ -1005,12 +1043,12 @@
       (channels/box (not (impl/closed? ch)))))))
 
 (defn remove>
-  "Deprecated - this function will be removed. Use transformer instead"
+  "Deprecated - this function will be removed. Use transducer instead"
   [p ch]
   (filter> (complement p) ch))
 
 (defn filter<
-  "Deprecated - this function will be removed. Use transformer instead"
+  "Deprecated - this function will be removed. Use transducer instead"
   ([p ch] (filter< p ch nil))
   ([p ch buf-or-n]
      (let [out (chan buf-or-n)]
@@ -1024,7 +1062,7 @@
        out)))
 
 (defn remove<
-  "Deprecated - this function will be removed. Use transformer instead"
+  "Deprecated - this function will be removed. Use transducer instead"
   ([p ch] (remove< p ch nil))
   ([p ch buf-or-n] (filter< (complement p) ch buf-or-n)))
 
@@ -1039,7 +1077,7 @@
               (recur)))))))
 
 (defn mapcat<
-  "Deprecated - this function will be removed. Use transformer instead"
+  "Deprecated - this function will be removed. Use transducer instead"
   ([f in] (mapcat< f in nil))
   ([f in buf-or-n]
     (let [out (chan buf-or-n)]
@@ -1047,7 +1085,7 @@
       out)))
 
 (defn mapcat>
-  "Deprecated - this function will be removed. Use transformer instead"
+  "Deprecated - this function will be removed. Use transducer instead"
 
   ([f out] (mapcat> f out nil))
   ([f out buf-or-n]
@@ -1056,7 +1094,7 @@
        in)))
 
 (defn unique
- "Deprecated - this function will be removed. Use transformer instead"
+ "Deprecated - this function will be removed. Use transducer instead"
   ([ch]
      (unique ch nil))
   ([ch buf-or-n]
@@ -1073,7 +1111,7 @@
 
 
 (defn partition
-  "Deprecated - this function will be removed. Use transformer instead"
+  "Deprecated - this function will be removed. Use transducer instead"
   ([n ch]
      (partition n ch nil))
   ([n ch buf-or-n]
@@ -1097,7 +1135,7 @@
 
 
 (defn partition-by
-  "Deprecated - this function will be removed. Use transformer instead"
+  "Deprecated - this function will be removed. Use transducer instead"
   ([f ch]
      (partition-by f ch nil))
   ([f ch buf-or-n]
